/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */
/*
 * Format of an instruction in memory.
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1996, 2000 by Ralf Baechle
 * Copyright (C) 2006 by Thiemo Seufer
 * Copyright (C) 2012 MIPS Technologies, Inc.  All rights reserved.
 * Copyright (C) 2014 Imagination Technologies Ltd.
 */
#ifndef _UAPI_ASM_INST_H
#define _UAPI_ASM_INST_H

#include <asm/bitfield.h>

/* For LOONGARCH */
#if 1
enum reg0_op {
	tlbinv_op = 0x19208, gtlbinv_op=0x19208,
	tlbflush_op = 0x19209, gtlbflush_op=0x19209,
	tlbp_op = 0x1920a, gtlbp_op=0x1920a,
	tlbr_op = 0x1920b, gtlbr_op=0x1920b,
	tlbwi_op = 0x1920c, gtlbwi_op=0x1920c,
	tlbwr_op = 0x1920d, gtlbwr_op=0x1920d,
	eret_op = 0x1920e,
};

enum reg0i15_op {
	break_op = 0x54, dbgcall_op, syscall_op, hypcall_op,
	wait_op = 0xc91, dbar_op = 0x70e4, ibar_op,
};

enum reg1i20_op {
	lu12iw_op = 0xa, lu32id_op, pcaddi_op, pcalau12i_op,
	pcaddu12i_op, pcaddu18i_op,
};

enum reg1i21_op {
	beqz_op = 0x10, bnez_op, bceqz_op, bcnez_op=0x12, jiscr0_op=0x12, jiscr1_op=0x12,
	jirl_op, b_op, bl_op, beq_op, bne_op, blt_op, bge_op, bltu_op, bgeu_op,
};

enum reg2_op {
	gr2scr_op = 0x2, scr2gr_op, clow_op,
	clzw_op, ctow_op, ctzw_op, clod_op,
	clzd_op, ctod_op, ctzd_op, revb2h_op,
	revb4h_op, revb2w_op, revbd_op, revh2w_op,
	revhd_op, bitrev4b_op, bitrev8b_op, bitrevw_op,
	bitrevd_op, extwh_op, extwb_op, rdtimelw_op,
	rdtimehw_op, rdtimed_op, cpucfg_op,
	iocsrrdb_op = 0x19200, iocsrrdh_op, iocsrrdw_op, iocsrrdd_op,
	iocsrwrb_op, iocsrwrh_op, iocsrwrw_op, iocsrwrd_op,
	movgr2fcsr_op = 0x4530, movfcsr2gr_op = 0x4532,
	movgr2cf_op = 0x4536, movcf2gr_op = 0x4537,
};

enum reg2ui3_op {
	rotrib_op = 0x261, rcrib_op = 0x281,
};

enum reg2ui4_op {
	rotrih_op = 0x131, rcrih_op = 0x141,
};

enum reg2ui5_op {
	slliw_op = 0x81, srliw_op = 0x89, sraiw_op = 0x91, rotriw_op = 0x99,
	rcriw_op = 0xa1,
};

enum reg2ui6_op {
	sllid_op = 0x41, srlid_op = 0x45, sraid_op = 0x49, rotrid_op = 0x4d,
	rcrid_op = 0x51,
};

enum reg2lsbw_op {
	bstrinsw_op = 0x3, bstrpickw_op = 0x3,
};

enum reg2lsbd_op {
	bstrinsd_op = 0x2, bstrpickd_op = 0x3,
};

enum reg2i8_op {
	lddir_op = 0x190, ldpte_op,
};

enum reg2i8idx1_op {
	vstelmd_op = 0x622,
};

enum reg2i8idx2_op {
	vstelmw_op = 0x312, xvstelmd_op = 0x331,
};

enum reg2i8idx3_op {
	vstelmh_op = 0x18a, xvstelmw_op = 0x199,
};

enum reg2i8idx4_op {
	vstelmb_op = 0xc6, xvstelmh_op = 0xcd,
};

enum reg2i8idx5_op {
	xvstelmb_op = 0x67,
};

enum reg2i9_op {
	vldrepld_op = 0x602, xvldrepld_op = 0x642,
};

enum reg2i10_op {
	vldreplw_op = 0x302, xvldreplw_op = 0x322,
};

enum reg2i11_op {
	vldreplh_op = 0x182, xvldreplh_op = 0x192,
};

enum reg2i12_op {
	slti_op = 0x8, sltui_op, addiw_op, addid_op,
	lu52id_op, andi_op, ori_op, xori_op,
	cache_op = 0x18, xvldreplb_op = 0xca,
	ldb_op = 0xa0, ldh_op, ldw_op, ldd_op, stb_op, sth_op,
	stw_op, std_op, ldbu_op, ldhu_op, ldwu_op, preld_op,
	flds_op, fsts_op, fldd_op, fstd_op, vld_op, vst_op, xvld_op,
	xvst_op, ldlw_op = 0xb8, ldrw_op, ldld_op, ldrd_op, stlw_op,
	strw_op, stld_op, strd_op, vldreplb_op = 0xc2,
};

enum reg2i14_op {
	llw_op = 0x20, scw_op, lld_op, scd_op, ldptrw_op, stptrw_op,
	ldptrd_op, stptrd_op,
};

enum reg2i16_op {
	addu16id_op = 0x4,
};

enum reg2csr_op {
	csrrd_op = 0x4, csrwr_op = 0x4, csrxchg_op = 0x4,
	gcsrrd_op = 0x5, gcsrwr_op = 0x5, gcsrxchg_op = 0x5,
};

enum reg3_op {
	asrtled_op = 0x2, asrtgtd_op,
	addw_op = 0x20, addd_op, subw_op, subd_op,
	slt_op, sltu_op, maskeqz_op, masknez_op,
	nor_op, and_op, or_op, xor_op, orn_op,
	andn_op, sllw_op, srlw_op, sraw_op, slld_op,
	srld_op, srad_op, rotrb_op, rotrh_op,
	rotrw_op, rotrd_op, mulw_op, mulhw_op,
	mulhwu_op, muld_op, mulhd_op, mulhdu_op,
	mulwdw_op, mulwdwu_op, divw_op, modw_op,
	divwu_op, modwu_op, divd_op, modd_op,
	divdu_op, moddu_op, crcwbw_op,
	crcwhw_op, crcwww_op, crcwdw_op, crccwbw_op,
	crccwhw_op, crccwww_op, crccwdw_op, addu12iw_op,
	addu12id_op,
	adcb_op = 0x60, adch_op, adcw_op, adcd_op,
	sbcb_op, sbch_op, sbcw_op, sbcd_op,
	rcrb_op, rcrh_op, rcrw_op, rcrd_op,
	ldxb_op = 0x7000, ldxh_op = 0x7008, ldxw_op = 0x7010, ldxd_op = 0x7018,
	stxb_op = 0x7020, stxh_op = 0x7028, stxw_op = 0x7030, stxd_op = 0x7038,
	ldxbu_op = 0x7040, ldxhu_op = 7048, ldxwu_op = 0x7050,
	preldx_op = 0x7058, fldxs_op = 0x7060, fldxd_op = 0x7068,
	fstxs_op = 0x7070, fstxd_op = 0x7078, vldx_op = 0x7080,
	vstx_op = 0x7088, xvldx_op = 0x7090, xvstx_op = 0x7098,
	amswapw_op = 0x70c0, amswapd_op, amaddw_op, amaddd_op, amandw_op,
	amandd_op, amorw_op, amord_op, amxorw_op, amxord_op, ammaxw_op,
	ammaxd_op, amminw_op, ammind_op, ammaxwu_op, ammaxdu_op,
	amminwu_op, ammindu_op, amswap_dbw_op, amswap_dbd_op, amadd_dbw_op,
	amadd_dbd_op, amand_dbw_op, amand_dbd_op, amor_dbw_op, amor_dbd_op,
	amxor_dbw_op, amxor_dbd_op, ammax_dbw_op, ammax_dbd_op, ammin_dbw_op,
	ammin_dbd_op, ammax_dbwu_op, ammax_dbdu_op, ammin_dbwu_op,
	ammin_dbdu_op, fldgts_op = 0x70e8, fldgtd_op,
	fldles_op, fldled_op, fstgts_op, fstgtd_op, fstles_op, fstled_op,
	ldgtb_op, ldgth_op, ldgtw_op, ldgtd_op, ldleb_op, ldleh_op, ldlew_op,
	ldled_op, stgtb_op, stgth_op, stgtw_op, stgtd_op, stleb_op, stleh_op,
	stlew_op, stled_op,
};

enum reg3sa2_op {
	alslw_op = 0x2, alslwu_op, bytepickw_op, alsld_op = 0x16,

};

enum reg3sa3_op {
	bytepickd_op = 0x3,
};
#endif /* end for LOONGARCH */

/*
 * func field of spec2 opcode.
 */
enum spec2_op {
	madd_op, maddu_op, mul_op, spec2_3_unused_op,
	msub_op, msubu_op, /* more unused ops */
	clz_op = 0x20, clo_op,
	dclz_op = 0x24, dclo_op,
	sdbpp_op = 0x3f
};

/*
 * func field of spec3 opcode.
 */
enum spec3_op {
	ext_op, dextm_op, dextu_op, dext_op,
	ins_op, dinsm_op, dinsu_op, dins_op,
	yield_op  = 0x09, lx_op     = 0x0a,
	lwle_op   = 0x19, lwre_op   = 0x1a,
	cachee_op = 0x1b, sbe_op    = 0x1c,
	she_op    = 0x1d, sce_op    = 0x1e,
	swe_op    = 0x1f, bshfl_op  = 0x20,
	swle_op   = 0x21, swre_op   = 0x22,
	prefe_op  = 0x23, dbshfl_op = 0x24,
	cache6_op = 0x25, sc6_op    = 0x26,
	scd6_op   = 0x27, lbue_op   = 0x28,
	lhue_op   = 0x29, lbe_op    = 0x2c,
	lhe_op    = 0x2d, lle_op    = 0x2e,
	lwe_op    = 0x2f, pref6_op  = 0x35,
	ll6_op    = 0x36, lld6_op   = 0x37,
	rdhwr_op  = 0x3b
};

/*
 * Bits 10-6 minor opcode for r6 spec mult/div encodings
 */
enum mult_op {
	mult_mult_op = 0x0,
	mult_mul_op = 0x2,
	mult_muh_op = 0x3,
};
enum multu_op {
	multu_multu_op = 0x0,
	multu_mulu_op = 0x2,
	multu_muhu_op = 0x3,
};
enum div_op {
	div_div_op = 0x0,
	div_div6_op = 0x2,
	div_mod_op = 0x3,
};
enum divu_op {
	divu_divu_op = 0x0,
	divu_divu6_op = 0x2,
	divu_modu_op = 0x3,
};
enum dmult_op {
	dmult_dmult_op = 0x0,
	dmult_dmul_op = 0x2,
	dmult_dmuh_op = 0x3,
};
enum dmultu_op {
	dmultu_dmultu_op = 0x0,
	dmultu_dmulu_op = 0x2,
	dmultu_dmuhu_op = 0x3,
};
enum ddiv_op {
	ddiv_ddiv_op = 0x0,
	ddiv_ddiv6_op = 0x2,
	ddiv_dmod_op = 0x3,
};
enum ddivu_op {
	ddivu_ddivu_op = 0x0,
	ddivu_ddivu6_op = 0x2,
	ddivu_dmodu_op = 0x3,
};

/*
 * func field of spec opcode.
 */
enum swc2_op {
	gssq_op = 0x20,
};

/*
 * func field of spec opcode.
 */
enum lwc2_op {
	gslq_op = 0x20,
};

/*
 * rt field of bcond opcodes.
 */
enum rt_op {
	bltz_op, bgez_op, bltzl_op, bgezl_op,
	spimi_op, unused_rt_op_0x05, unused_rt_op_0x06, unused_rt_op_0x07,
	tgei_op, tgeiu_op, tlti_op, tltiu_op,
	teqi_op, unused_0x0d_rt_op, tnei_op, unused_0x0f_rt_op,
	bltzal_op, bgezal_op, bltzall_op, bgezall_op,
	rt_op_0x14, rt_op_0x15, rt_op_0x16, rt_op_0x17,
	rt_op_0x18, rt_op_0x19, rt_op_0x1a, rt_op_0x1b,
	bposge32_op, rt_op_0x1d, rt_op_0x1e, synci_op
};

/*
 * rs field of cop opcodes.
 */
enum cop_op {
	mfc_op	      = 0x00, dmfc_op	    = 0x01,
	cfc_op	      = 0x02, mfhc0_op	    = 0x02,
	mfhc_op       = 0x03, mtc_op	    = 0x04,
	dmtc_op	      = 0x05, ctc_op	    = 0x06,
	mthc0_op      = 0x06, mthc_op	    = 0x07,
	bc_op	      = 0x08, bc1eqz_op     = 0x09,
	mfmc0_op      = 0x0b, bc1nez_op     = 0x0d,
	wrpgpr_op     = 0x0e, cop_op	    = 0x10,
	copm_op	      = 0x18
};

/*
 * rt field of cop.bc_op opcodes
 */
enum bcop_op {
	bcf_op, bct_op, bcfl_op, bctl_op
};

/*
 * fmt field of cop1 opcodes.
 */
enum cop1_fmt {
	s_fmt, d_fmt, e_fmt, q_fmt,
	w_fmt, l_fmt
};

/*
 * func field of cop1 instructions using d, s or w format.
 */
enum cop1_sdw_func {
	fadd_op	     =	0x00, fsub_op	   =  0x01,
	fmul_op	     =	0x02, fdiv_op	   =  0x03,
	fsqrt_op     =	0x04, fabs_op	   =  0x05,
	fmov_op	     =	0x06, fneg_op	   =  0x07,
	froundl_op   =	0x08, ftruncl_op   =  0x09,
	fceill_op    =	0x0a, ffloorl_op   =  0x0b,
	fround_op    =	0x0c, ftrunc_op	   =  0x0d,
	fceil_op     =	0x0e, ffloor_op	   =  0x0f,
	fsel_op      =  0x10,
	fmovc_op     =	0x11, fmovz_op	   =  0x12,
	fmovn_op     =	0x13, fseleqz_op   =  0x14,
	frecip_op    =  0x15, frsqrt_op    =  0x16,
	fselnez_op   =  0x17, fmaddf_op    =  0x18,
	fmsubf_op    =  0x19, frint_op     =  0x1a,
	fclass_op    =  0x1b, fmin_op      =  0x1c,
	fmina_op     =  0x1d, fmax_op      =  0x1e,
	fmaxa_op     =  0x1f, fcvts_op     =  0x20,
	fcvtd_op     =	0x21, fcvte_op	   =  0x22,
	fcvtw_op     =	0x24, fcvtl_op	   =  0x25,
	fcmp_op	     =	0x30
};

/*
 * func field of cop1x opcodes (LOONGARCH IV).
 */
enum cop1x_func {
	lwxc1_op     =	0x00, ldxc1_op	   =  0x01,
	swxc1_op     =  0x08, sdxc1_op	   =  0x09,
	pfetch_op    =	0x0f, madd_s_op	   =  0x20,
	madd_d_op    =	0x21, madd_e_op	   =  0x22,
	msub_s_op    =	0x28, msub_d_op	   =  0x29,
	msub_e_op    =	0x2a, nmadd_s_op   =  0x30,
	nmadd_d_op   =	0x31, nmadd_e_op   =  0x32,
	nmsub_s_op   =	0x38, nmsub_d_op   =  0x39,
	nmsub_e_op   =	0x3a
};

/*
 * func field for mad opcodes (LOONGARCH IV).
 */
enum mad_func {
	madd_fp_op	= 0x08, msub_fp_op	= 0x0a,
	nmadd_fp_op	= 0x0c, nmsub_fp_op	= 0x0e
};

/*
 * func field for special3 lx opcodes (Cavium Octeon).
 */
enum lx_func {
	lwx_op	= 0x00,
	lhx_op	= 0x04,
	lbux_op = 0x06,
	ldx_op	= 0x08,
	lwux_op = 0x10,
	lhux_op = 0x14,
	lbx_op	= 0x16,
};

/*
 * BSHFL opcodes
 */
enum bshfl_func {
	wsbh_op = 0x2,
	seb_op  = 0x10,
	seh_op  = 0x18,
};

/*
 * DBSHFL opcodes
 */
enum dbshfl_func {
	dsbh_op = 0x2,
	dshd_op = 0x5,
};

/*
 * MSA minor opcodes.
 */
enum msa_func {
	msa_elm_op = 0x19,
};

/*
 * MSA ELM opcodes.
 */
enum msa_elm {
	msa_ctc_op = 0x3e,
	msa_cfc_op = 0x7e,
};

/*
 * func field for MSA MI10 format.
 */
enum msa_mi10_func {
	msa_ld_op = 8,
	msa_st_op = 9,
};

/*
 * MSA 2 bit format fields.
 */
enum msa_2b_fmt {
	msa_fmt_b = 0,
	msa_fmt_h = 1,
	msa_fmt_w = 2,
	msa_fmt_d = 3,
};

#if 1
struct reg2_format {
	__BITFIELD_FIELD(unsigned int opcode : 22, /* Jump format */
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;)))
};

struct reg2ui3_format {
	__BITFIELD_FIELD(unsigned int opcode : 19, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 3,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))
};

struct reg2ui4_format {
	__BITFIELD_FIELD(unsigned int opcode : 18, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 4,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))
};

struct reg2ui5_format {
	__BITFIELD_FIELD(unsigned int opcode : 17, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 5,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))
};

struct reg2ui6_format {
	__BITFIELD_FIELD(unsigned int opcode : 16, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 6,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))
};

struct reg2lsbw_format {
	__BITFIELD_FIELD(unsigned int opcode : 11, /* Jump format */
	__BITFIELD_FIELD(unsigned int msbw : 5,
	__BITFIELD_FIELD(unsigned int op : 1,
	__BITFIELD_FIELD(unsigned int lsbw : 5,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))))
};

struct reg2lsbd_format {
	__BITFIELD_FIELD(unsigned int opcode : 10, /* Jump format */
	__BITFIELD_FIELD(unsigned int msbd : 6,
	__BITFIELD_FIELD(unsigned int lsbd : 6,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;)))))
};

struct reg3_format {
	__BITFIELD_FIELD(unsigned int opcode : 17, /* Jump format */
	__BITFIELD_FIELD(unsigned int rk : 5,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))
};

struct reg3sa2_format {
	__BITFIELD_FIELD(unsigned int opcode : 15, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 2,
	__BITFIELD_FIELD(unsigned int rk : 5,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;)))))
};

struct reg3sa3_format {
	__BITFIELD_FIELD(unsigned int opcode : 14, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 3,
	__BITFIELD_FIELD(unsigned int rk : 5,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;)))))
};

struct reg3sa4_format {
	__BITFIELD_FIELD(unsigned int opcode : 13, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 4,
	__BITFIELD_FIELD(unsigned int rk : 5,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;)))))
};

struct reg4_format {
	__BITFIELD_FIELD(unsigned int opcode : 12, /* Jump format */
	__BITFIELD_FIELD(unsigned int fa : 5,
	__BITFIELD_FIELD(unsigned int fk : 5,
	__BITFIELD_FIELD(unsigned int fj : 5,
	__BITFIELD_FIELD(unsigned int fd : 5,
	;)))))
};

struct reg2i8_format {
	__BITFIELD_FIELD(unsigned int opcode : 14, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 8,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))
};

struct reg2i8idx1_format {
	__BITFIELD_FIELD(unsigned int opcode : 13, /* Jump format */
	__BITFIELD_FIELD(unsigned int idx : 1,
	__BITFIELD_FIELD(unsigned int simmediate : 8,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;)))))
};

struct reg2i8idx2_format {
	__BITFIELD_FIELD(unsigned int opcode : 12, /* Jump format */
	__BITFIELD_FIELD(unsigned int idx : 2,
	__BITFIELD_FIELD(unsigned int simmediate : 8,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;)))))
};

struct reg2i8idx3_format {
	__BITFIELD_FIELD(unsigned int opcode : 11, /* Jump format */
	__BITFIELD_FIELD(unsigned int idx : 3,
	__BITFIELD_FIELD(unsigned int simmediate : 8,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;)))))
};

struct reg2i8idx4_format {
	__BITFIELD_FIELD(unsigned int opcode : 10, /* Jump format */
	__BITFIELD_FIELD(unsigned int idx : 4,
	__BITFIELD_FIELD(unsigned int simmediate : 8,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;)))))
};

struct reg2i8idx5_format {
	__BITFIELD_FIELD(unsigned int opcode : 9, /* Jump format */
	__BITFIELD_FIELD(unsigned int idx : 5,
	__BITFIELD_FIELD(unsigned int simmediate : 8,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;)))))
};

struct reg2i9_format {
	__BITFIELD_FIELD(unsigned int opcode : 13, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 9,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))
};

struct reg2i10_format {
	__BITFIELD_FIELD(unsigned int opcode : 12, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 10,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))
};

struct reg2i11_format {
	__BITFIELD_FIELD(unsigned int opcode : 11, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 11,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))
};

struct reg2i12_format {
	__BITFIELD_FIELD(unsigned int opcode : 10, /* Cache format */
	__BITFIELD_FIELD(signed int simmediate : 12,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))
};

struct reg2u12_format {
	__BITFIELD_FIELD(unsigned int opcode : 10, /* Cache format */
	__BITFIELD_FIELD(unsigned int simmediate : 12,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))
};

struct reg2i14_format {
	__BITFIELD_FIELD(unsigned int opcode : 8, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 14,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))
};

struct reg2i16_format {
	__BITFIELD_FIELD(unsigned int opcode : 6, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 16,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))
};

struct reg2csr_format {
	__BITFIELD_FIELD(unsigned int opcode : 8, /* Jump format */
	__BITFIELD_FIELD(unsigned int csr : 14,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;))))
};

struct reg1i21_format {
	__BITFIELD_FIELD(unsigned int opcode : 6, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate_l : 16,
	__BITFIELD_FIELD(unsigned int rj : 5,
	__BITFIELD_FIELD(unsigned int simmediate_h  : 5,
	;))))
};

struct reg1i20_format {
	__BITFIELD_FIELD(unsigned int opcode : 7, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 20,
	__BITFIELD_FIELD(unsigned int rd : 5,
	;)))
};

struct i15_format {
	__BITFIELD_FIELD(unsigned int opcode : 17, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 15,
	;))
};

struct i26_format {
	__BITFIELD_FIELD(unsigned int opcode : 6, /* Jump format */
	__BITFIELD_FIELD(unsigned int simmediate : 26,
	;))
};
#endif

struct j_format {
	__BITFIELD_FIELD(unsigned int opcode : 6, /* Jump format */
	__BITFIELD_FIELD(unsigned int target : 26,
	;))
};

struct i_format {			/* signed immediate format */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int rs : 5,
	__BITFIELD_FIELD(unsigned int rt : 5,
	__BITFIELD_FIELD(signed int simmediate : 16,
	;))))
};

struct u_format {			/* unsigned immediate format */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int rs : 5,
	__BITFIELD_FIELD(unsigned int rt : 5,
	__BITFIELD_FIELD(unsigned int uimmediate : 16,
	;))))
};

struct c_format {			/* Cache (>= R6000) format */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int rs : 5,
	__BITFIELD_FIELD(unsigned int c_op : 3,
	__BITFIELD_FIELD(unsigned int cache : 2,
	__BITFIELD_FIELD(unsigned int simmediate : 16,
	;)))))
};

struct r_format {			/* Register format */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int rs : 5,
	__BITFIELD_FIELD(unsigned int rt : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	__BITFIELD_FIELD(unsigned int re : 5,
	__BITFIELD_FIELD(unsigned int func : 6,
	;))))))
};

struct c0r_format {			/* C0 register format */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int rs : 5,
	__BITFIELD_FIELD(unsigned int rt : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	__BITFIELD_FIELD(unsigned int z: 8,
	__BITFIELD_FIELD(unsigned int sel : 3,
	;))))))
};

struct mfmc0_format {			/* MFMC0 register format */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int rs : 5,
	__BITFIELD_FIELD(unsigned int rt : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	__BITFIELD_FIELD(unsigned int re : 5,
	__BITFIELD_FIELD(unsigned int sc : 1,
	__BITFIELD_FIELD(unsigned int : 2,
	__BITFIELD_FIELD(unsigned int sel : 3,
	;))))))))
};

struct co_format {			/* C0 CO format */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int co : 1,
	__BITFIELD_FIELD(unsigned int code : 19,
	__BITFIELD_FIELD(unsigned int func : 6,
	;))))
};

struct p_format {		/* Performance counter format (R10000) */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int rs : 5,
	__BITFIELD_FIELD(unsigned int rt : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	__BITFIELD_FIELD(unsigned int re : 5,
	__BITFIELD_FIELD(unsigned int func : 6,
	;))))))
};

struct f_format {			/* FPU register format */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int : 1,
	__BITFIELD_FIELD(unsigned int fmt : 4,
	__BITFIELD_FIELD(unsigned int rt : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	__BITFIELD_FIELD(unsigned int re : 5,
	__BITFIELD_FIELD(unsigned int func : 6,
	;)))))))
};

struct ma_format {		/* FPU multiply and add format (LOONGARCH IV) */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int fr : 5,
	__BITFIELD_FIELD(unsigned int ft : 5,
	__BITFIELD_FIELD(unsigned int fs : 5,
	__BITFIELD_FIELD(unsigned int fd : 5,
	__BITFIELD_FIELD(unsigned int func : 4,
	__BITFIELD_FIELD(unsigned int fmt : 2,
	;)))))))
};

struct b_format {			/* BREAK and SYSCALL */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int code : 20,
	__BITFIELD_FIELD(unsigned int func : 6,
	;)))
};

struct ps_format {			/* LOONGARCH-3D / paired single format */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int rs : 5,
	__BITFIELD_FIELD(unsigned int ft : 5,
	__BITFIELD_FIELD(unsigned int fs : 5,
	__BITFIELD_FIELD(unsigned int fd : 5,
	__BITFIELD_FIELD(unsigned int func : 6,
	;))))))
};

struct v_format {				/* MDMX vector format */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int sel : 4,
	__BITFIELD_FIELD(unsigned int fmt : 1,
	__BITFIELD_FIELD(unsigned int vt : 5,
	__BITFIELD_FIELD(unsigned int vs : 5,
	__BITFIELD_FIELD(unsigned int vd : 5,
	__BITFIELD_FIELD(unsigned int func : 6,
	;)))))))
};

struct msa_mi10_format {		/* MSA MI10 */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(signed int s10 : 10,
	__BITFIELD_FIELD(unsigned int rs : 5,
	__BITFIELD_FIELD(unsigned int wd : 5,
	__BITFIELD_FIELD(unsigned int func : 4,
	__BITFIELD_FIELD(unsigned int df : 2,
	;))))))
};

struct dsp_format {		/* SPEC3 DSP format instructions */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int base : 5,
	__BITFIELD_FIELD(unsigned int index : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	__BITFIELD_FIELD(unsigned int op : 5,
	__BITFIELD_FIELD(unsigned int func : 6,
	;))))))
};

struct spec3_format {   /* SPEC3 */
	__BITFIELD_FIELD(unsigned int opcode:6,
	__BITFIELD_FIELD(unsigned int rs:5,
	__BITFIELD_FIELD(unsigned int rt:5,
	__BITFIELD_FIELD(signed int simmediate:9,
	__BITFIELD_FIELD(unsigned int func:7,
	;)))))
};

struct fp6_format {		/* FPU madd and msub format (LOONGARCH IV) */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int fr : 5,
	__BITFIELD_FIELD(unsigned int ft : 5,
	__BITFIELD_FIELD(unsigned int fs : 5,
	__BITFIELD_FIELD(unsigned int fd : 5,
	__BITFIELD_FIELD(unsigned int func : 6,
	;))))))
};

/*
 * Loongson-3 overridden COP2 instruction formats (32-bit length)
 */
struct loongson3_lswc2_format {	/* Loongson-3 overridden lwc2/swc2 Load/Store format */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int base : 5,
	__BITFIELD_FIELD(unsigned int rt : 5,
	__BITFIELD_FIELD(unsigned int fr : 1,
	__BITFIELD_FIELD(unsigned int offset : 9,
	__BITFIELD_FIELD(unsigned int ls : 1,
	__BITFIELD_FIELD(unsigned int rq : 5,
	;)))))))
};

struct loongson3_lsdc2_format {	/* Loongson-3 overridden ldc2/sdc2 Load/Store format */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int base : 5,
	__BITFIELD_FIELD(unsigned int rt : 5,
	__BITFIELD_FIELD(unsigned int index : 5,
	__BITFIELD_FIELD(unsigned int offset : 8,
	__BITFIELD_FIELD(unsigned int opcode1 : 3,
	;))))))
};

struct loongson3_lscsr_format {		/* Loongson-3 compatiable csr read/write format */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int rs : 5,
	__BITFIELD_FIELD(unsigned int fr : 5,
	__BITFIELD_FIELD(unsigned int rd : 5,
	__BITFIELD_FIELD(unsigned int fd : 5,
	__BITFIELD_FIELD(unsigned int func : 6,
	;))))))
};

struct loongarch_dbg_format {		/* Loongson-3 compatiable csr read/write format */
	__BITFIELD_FIELD(unsigned int opcode : 6,
	__BITFIELD_FIELD(unsigned int rs : 5,
	__BITFIELD_FIELD(unsigned int rt : 5,
	__BITFIELD_FIELD(unsigned int type : 4,
	__BITFIELD_FIELD(unsigned int csr : 12,
	;)))))
};

union loongarch_instruction {
	unsigned int word;
	unsigned short halfword[2];
	unsigned char byte[4];
#if 1
	struct reg2_format reg2_format;
	struct reg2ui3_format reg2ui3_format;
	struct reg2ui4_format reg2ui4_format;
	struct reg2ui5_format reg2ui5_format;
	struct reg2ui6_format reg2ui6_format;
	struct reg2lsbw_format reg2lsbw_format;
	struct reg2lsbd_format reg2lsbd_format;
	struct reg3_format reg3_format;
	struct reg3sa2_format reg3sa2_format;
	struct reg3sa3_format reg3sa3_format;
	struct reg3sa4_format reg3sa4_format;
	struct reg4_format reg4_format;
	struct reg2i8_format reg2i8_format;
	struct reg2i8idx1_format reg2i8idx1_format;
	struct reg2i8idx2_format reg2i8idx2_format;
	struct reg2i8idx3_format reg2i8idx3_format;
	struct reg2i8idx4_format reg2i8idx4_format;
	struct reg2i8idx5_format reg2i8idx5_format;
	struct reg2i9_format reg2i9_format;
	struct reg2i10_format reg2i10_format;
	struct reg2i11_format reg2i11_format;
	struct reg2i12_format reg2i12_format;
	struct reg2i14_format reg2i14_format;
	struct reg2i16_format reg2i16_format;
	struct reg2csr_format reg2csr_format;
	struct reg1i21_format reg1i21_format;
	struct reg1i20_format reg1i20_format;
	struct i15_format i15_format;
	struct i26_format i26_format;
#endif
	struct j_format j_format;
	struct i_format i_format;
	struct u_format u_format;
	struct c_format c_format;
	struct r_format r_format;
	struct c0r_format c0r_format;
	struct mfmc0_format mfmc0_format;
	struct co_format co_format;
	struct p_format p_format;
	struct f_format f_format;
	struct ma_format ma_format;
	struct msa_mi10_format msa_mi10_format;
	struct b_format b_format;
	struct ps_format ps_format;
	struct v_format v_format;
	struct dsp_format dsp_format;
	struct spec3_format spec3_format;
	struct fp6_format fp6_format;
	struct loongson3_lswc2_format loongson3_lswc2_format;
	struct loongson3_lsdc2_format loongson3_lsdc2_format;
	struct loongson3_lscsr_format loongson3_lscsr_format;
	struct loongarch_dbg_format loongarch_dbg_format;
};

#endif /* _UAPI_ASM_INST_H */
